/*
 * Copyright (c) 2022 Virtuozzo International GmbH.
 * Author: Andrey Zhadchenko <andrey.zhadchenko@virtuozzo.com>
 *
 * vhost-blk is host kernel accelerator for virtio-blk.
 *
 * This work is licensed under the terms of the GNU LGPL, version 2 or later.
 * See the COPYING.LIB file in the top-level directory.
 */

#ifndef VHOST_BLK_H
#define VHOST_BLK_H

#include "standard-headers/linux/virtio_blk.h"
#include "qemu-common.h"
#include "hw/block/block.h"
#include "hw/virtio/vhost.h"
#include "sysemu/block-backend.h"

#define TYPE_VHOST_BLK "vhost-blk"
#define VHOST_BLK(obj) \
        OBJECT_CHECK(VHostBlk, (obj), TYPE_VHOST_BLK)

#define VHOST_BLK_AUTO_NUM_QUEUES UINT16_MAX
#define VHOST_BLK_MAX_QUEUES 8

typedef struct VhostBlkConf {
    BlockConf conf;
    uint16_t num_queues;
    uint16_t queue_size;
    uint16_t num_threads;
    char *serial;
} VhostBlkConf;

typedef struct VHostBlk {
    VirtIODevice parent_obj;
    VhostBlkConf conf;
    uint64_t host_features;
    uint64_t decided_features;
    struct virtio_blk_config blkcfg;
    int vhostfd;
    struct vhost_dev dev;
    bool vhost_started;
} VHostBlk;

#endif
