/*
 *  AUX IO Multiplexing
 *
 * Authors:
 *  Yuriy Pudgorodskiy, <yur@virtuozzo.com>
 *
 * This work is licensed under the terms of the GNU LGPL, version 2.1 or later.
 * See the COPYING.LIB file in the top-level directory.
 *
 */

#ifndef AUX_IO_H
#define AUX_IO_H

#include <stdint.h>

#define AUX_MAGIC_NUMBER 0x4B58B9CA

typedef struct AuxMessageHeader {
    uint32_t            magic;
    uint32_t            cid;
    uint32_t            length;
#ifdef DEBUG_AUX_IO
    uint32_t            seq;
#endif
} AuxMessageHeader;

typedef struct AuxMessage {
    AuxMessageHeader    header;
    char                data[0];
} AuxMessage;

typedef struct AuxIoChannel {
    guint  revent;
    guint  cid;
    GIOChannel *rchan;
    GIOChannel *wchan;
    GIOChannel **pchan;
    void (*cb)(const char *buffer, size_t size, void *cbdata);
    size_t size;
    size_t total;
    bool   overflow;

    char   name[32];
} AuxIoChannel;

typedef struct AuxMessageParser {
    AuxMessageHeader    header;
    size_t              header_len;
    size_t              data_len;
    int                 state;
} AuxMessageParser;

void aux_message_parser_feed(AuxMessageParser *parser, const char *buffer, size_t size);

void aux_add_channel(int fd, guint cid, GIOChannel **ch);
void aux_remove_channel(guint cid);

void aux_init(void);

#endif
